/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef GERQ_ASSOCIATOR_H
#define GERQ_ASSOCIATOR_H


#include "gerqutilsbase.h"

#include "assoadapter.h"
#include "datacontainer.h"
#include "dataobject.h"
#include <QMap>
#include <QMapIterator>
#include <QString>


namespace GerQ {


  typedef QHash<QString, DataObject*> AssoData;
  typedef QHash<QString, DataObject*>::iterator AssoDataIterator;


  class Associator {

    public:

      Associator(
        AssoAdapter *aAdapter, DataContainer *aCtrYin, DataContainer *aCtrYan);
      virtual ~Associator();

      AssoAdapter *getAdapter();

      void read(DataObject *aData, DataContainer *aContainer);
      void write(DataObject *aData, DataContainer *aContainer);

      static Associator *addDefaultAssociator(
        const QString &aTableName,
	const QString &aYinColumn, const QString &aYanColumn,
	DataContainer *aCtrYin, DataContainer *aCtrYan,
	void (*aAssociate)(DataObject*, DataObject*),
	void (*aDissociate)(DataObject*, DataObject*),
	void (*aGetYanData)(DataObject*, AssoData&),
	void (*aGetYinData)(DataObject*, AssoData&));
      static Associator *getDefaultAssociator(const QString &aTableName);

    protected:

      virtual void associate(DataObject *aYin, DataObject *aYan) = ABSTRACT;
      virtual void dissociate(DataObject *aYin, DataObject *aYan) = ABSTRACT;

      virtual void getYinData(DataObject *aYan, AssoData &aYinData) = ABSTRACT;
      virtual void getYanData(DataObject *aYin, AssoData &aYanData) = ABSTRACT;

    private:

      AssoAdapter *fAdapter;

      DataContainer *fCtrYin;
      DataContainer *fCtrYan;

      void readYin(DataObject *aYin);
      void readYan(DataObject *aYan);
      void writeYin(DataObject *aYin);
      void writeYan(DataObject *aYan);

      static QHash<QString, Associator*> fDefaultAssociators;

  }; // Associator


} // GerQ


#endif
