/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "dataobject.h"

#include "datacontainer.h"
#include "refcontainer.h"


namespace GerQ {

  DataObject::DataObject() {
    fID = 0;
    fContainer = 0;
  } // DataObject::DataObject

  DataObject::~DataObject() {
    /*
      On destruction, remove yourself from the container for your class as well
      as from the containers of all anchestor classes:
    */
    DataContainer *ctr = fContainer;
    while (ctr) {
      QString ID = fID.toString();
      if (ctr -> fData.contains(ID)) {
        ctr -> fData.remove(ID);
      } // if
      ctr = ctr -> getParent();
    } // while
  } // DataObject::~DataObject

  void DataObject::erase() {
    /*
      On erase, make sure to erase yourself from the container for your class
      as well as from the containers of all anchestor classes:
    */
    DataContainer *ctr = fContainer;
    while (ctr) {
      ctr -> getAdapter() -> erase(this);
      ctr = ctr -> getParent();
    } // while
    delete this;
  } // DataObject::erase

  InstContainer* DataObject::getContainer() {
    return fContainer;
  } // DataObject::getContainer

  QVariant DataObject::getID() {
    return fID;
  } // DataObject::getID

  bool DataObject::read() {
    return fContainer -> read(this);
  } // DataObject::read

  void DataObject::write() {
    fContainer -> write(this);
  } // DataObject::write

} // GerQ
