/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "sqlassoadapter.h"

#include <QSqlRecord>


namespace GerQ {

  SqlAssoAdapter::SqlAssoAdapter(
    const QString &aTableName,
    const QString &aYinColumn,
    const QString &aYanColumn
  ) {
    fTable = new QSqlTableModel();
    fTable -> setTable(aTableName);
    fYinColumn = aYinColumn;
    fYanColumn = aYanColumn;
  } // SqlAssoAdapter::SqlAssoAdapter

  SqlAssoAdapter::~SqlAssoAdapter() {
    delete fTable;
  } // SqlAssoAdapter::~SqlAssoAdapter

  void SqlAssoAdapter::deleteAssociation(
    DataObject *aYin, DataObject *aYan
  ) {
    fTable -> setFilter(getFullFilter(aYin -> getID(), aYan -> getID()));
    fTable -> select();
    if (fTable -> rowCount() == 1) {
      fTable -> removeRows(0, 1);
      fTable -> submitAll();
    } // if
  } // SqlAssoAdapter::deleteAssociation

  QString SqlAssoAdapter::getFilter(
    const QString &aCol, const QVariant &aID
  ) {
    switch (aID.type()) {
      case QVariant::Char:
      case QVariant::String:
        return QString().sprintf("%s = '%s'",
	  qPrintable(aCol), qPrintable(aID.toString()));
      default:
        return QString().sprintf("%s = %s",
	  qPrintable(aCol), qPrintable(aID.toString()));
    } // switch
  } // SqlAssoAdapter::getFilter

  QString SqlAssoAdapter::getFullFilter(
    QVariant aYinID, QVariant aYanID
  ) {
    return QString().sprintf("(%s) and (%s)",
      qPrintable(getFilter(fYinColumn, aYinID)),
      qPrintable(getFilter(fYanColumn, aYanID)));
  } // SqlAssoAdapter::getFullFilter

  QString SqlAssoAdapter::getTableName() {
    return fTable -> tableName();
  } // SqlAssoAdapter::getTableName

  QString SqlAssoAdapter::getYanColumn() {
    return fYanColumn;
  } // SqlAssoAdapter::getYanColumn

  void SqlAssoAdapter::getYanIDs(DataObject *aYin, AssoIDs &aIDs) {
    QVariant IDYin = aYin -> getID();
    fTable -> setFilter(getFilter(fYinColumn, IDYin));
    fTable -> select();
    int n = fTable -> rowCount();
    for (int i = 0; i < n; i++) {
      QString IDYan = fTable -> record(i).value(fYanColumn).toString();
      aIDs.insert(IDYan, IDYan);
    } // for
  } // SqlAssoAdapter::getYanIDs

  QString SqlAssoAdapter::getYinColumn() {
    return fYinColumn;
  } // SqlAssoAdapter::getYinColumn

  void SqlAssoAdapter::getYinIDs(DataObject *aYan, AssoIDs &aIDs) {
    QVariant IDYan = aYan -> getID();
    fTable -> setFilter(getFilter(fYanColumn, IDYan));
    fTable -> select();
    int n = fTable -> rowCount();
    for (int i = 0; i < n; i++) {
      QString IDYin = fTable -> record(i).value(fYinColumn).toString();
      aIDs.insert(IDYin, IDYin);
    } // for
  } // SqlAssoAdapter::getYinIDs

  void SqlAssoAdapter::insertAssociation(
    DataObject *aYin, DataObject *aYan
  ) {
    QVariant IDYin = aYin -> getID();
    QVariant IDYan = aYan -> getID();
    fTable -> setFilter(getFullFilter(IDYin, IDYan));
    fTable -> select();
    if (fTable -> rowCount() == 0) {
      fTable -> insertRows(0, 1);
      int iFdYin = fTable -> fieldIndex(fYinColumn);
      int iFdYan = fTable -> fieldIndex(fYanColumn);
      fTable -> setData(fTable -> index(0, iFdYin), IDYin);
      fTable -> setData(fTable -> index(0, iFdYan), IDYan);
      fTable -> submitAll();
    } // if
  } // SqlAssoAdapter::insertAssociation

} // GerQ
