/*
  This file is part of GerQUtils.

  GerQUtils is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  GerQUtils is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
  A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  GerQUtils.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef GERQ_SQLDATAADAPTER_H
#define GERQ_SQLDATAADAPTER_H


#include "gerqutilsbase.h"

#include "dataadapter.h"
#include <QSqlTableModel>


namespace GerQ {


  class SqlDataAdapter: public DataAdapter {

    public:

      SqlDataAdapter(const QString &aTableName, const QString &aIDColumn);
      virtual ~SqlDataAdapter();

      virtual void reload();
      virtual void readAllIDs(QList<QVariant> &aIDs);

      virtual bool read(DataObject *aData);
      virtual void write(DataObject *aData);
      virtual void erase(DataObject *aData);

      QString getTableName();
      QString getIDColumn();

      QVariant getField(const QString &aName);
      void setField(const QString &aName, const QVariant &aValue);

    protected:

      virtual void doRead(DataObject *aData, SqlDataAdapter *aSql) = ABSTRACT;
      virtual void doWrite(
        DataObject *aData, SqlDataAdapter *aSql) = ABSTRACT;

      QSqlTableModel *getTable();

    private:

      QSqlTableModel *fTable;
      QString fIDColumn;

      void gotoRecord(DataObject *aData);

  }; // SqlDataAdapter


} // GerQ


#endif
